from audio import KEY_ARTIST
from audio import KEY_TITLE
from audio import KEY_YEAR
from audio import KEY_ALBUM
from audio import KEY_TRACK
from audio import KEY_GENRE
from audio import KEY_COMMENT
from audio import track_info
import CDDB
import DiscID
import logging
import subprocess


class compact_disc(object):
    def __init__(self, device, track_comment=''):
        self._device = device
        self._track_comment = track_comment
        self.disc_id = None
        self.read_disc_data()

    def __init_vars__(self):
        self._number_of_tracks = 0
        self._tracks = list()

    def read_disc_data(self):
        cdrom = DiscID.open(self._device)
        try:
            old_did = self.disc_id
            self.disc_id = DiscID.disc_id(cdrom)
            logging.debug('DISC-ID = %s', self.disc_id)
        finally:
            cdrom.close()
        if old_did != self.disc_id:
            self.process_cddb_data()

    def process_cddb_data(self):
        self.__init_vars__()
        try:
            query_info = CDDB.query(self.disc_id)[1]
            if type(query_info) == list:
                query_info = query_info[0]
            cddb_data = CDDB.read(query_info['category'], query_info['disc_id'])[1]
        except:
            cddb_data = None
        finally:
            self._number_of_tracks = self.disc_id[1]
            genre = cddb_data['DGENRE']
            artist, album = [txt.strip() for txt in cddb_data['DTITLE'].split('/')]
            year = int(cddb_data['DYEAR'])
            logging.debug('Number of Tracks = %d', self._number_of_tracks)
            logging.debug('Artist = %s', artist)
            logging.debug('Album = %s', album)
            logging.debug('Genre = %s', genre)
            logging.debug('Year = %d', year)
            for no in range(0, self._number_of_tracks):
                self._tracks.append(track_info(artist, album, genre, year, cddb_data['TTITLE%d' % no], no + 1, self._track_comment))
                logging.debug('Track%2d = %s', no + 1, cddb_data['TTITLE%d' % no])

    def rip(self, track_no, filename, progress_callback, *args, **kwargs):
        self.read_disc_data()
        rip_track(self._device, track_no, filename, progress_callback, *args, **kwargs)

    def get_track_data(self, track_no):
        if len(self._tracks) >= track_no:
            return self._tracks[track_no - 1]
        else:
            return dict()

    def __str__(self):
        rv = "Number of Tracks: %d\n" % self._number_of_tracks
        if self._number_of_tracks > 0:
            rv += "Artist: %s\n" % self._tracks[0].get(KEY_ARTIST)
            rv += "Album: %s\n" % self._tracks[0].get(KEY_ALBUM)
            rv += "Genre: %s\n" % self._tracks[0].get(KEY_GENRE)
            rv += "Year: %d\n" % self._tracks[0].get(KEY_YEAR)
            rv += "Comment: %d\n" % self._tracks[0].get(KEY_COMMENT)
            for no in range(0, self._number_of_tracks):
                rv += '    %3d: %s\n' % (self._tracks[no][KEY_TRACK], self._tracks[no][KEY_TITLE])
        return rv


def rip_track(device, track_no, filename, progress_callback, *args, **kwargs):
    cdp_secstarttag = 'sector'
    cdp_secendtag = '('
    cdp_starttag = '[wrote] @'
    cdp_endtag = '\n'

    cmd = 'cdparanoia --stderr-progress -d %(device)s %(track_no)d %(filename)s' % {'device': device, 'track_no': track_no, 'filename': filename}
    p = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
    line = ''
    startsec = None
    endsec = None
    startwrote = None
    progress = None
    while True:
        ch = p.stdout.read(1)
        if len(ch) == 0:
            break
        line += ch
        if line.find(cdp_secstarttag) > -1:
            line = line[line.find(cdp_secstarttag):]
            if line.find(cdp_secstarttag) > -1 and line.find(cdp_secendtag) > line.find(cdp_secstarttag):
                if startsec is None:
                    startsec = float(line[line.find(cdp_secstarttag) + len(cdp_secstarttag):line.find(cdp_secendtag)])
                    logging.debug('secstart: %s', str(startsec))
                elif endsec is None:
                    endsec = float(line[line.find(cdp_secstarttag) + len(cdp_secstarttag):line.find(cdp_secendtag)])
                    logging.debug('secend: %s', str(endsec))
                line = line[line.find(cdp_secendtag) + len(cdp_secendtag):]
        elif line.find(cdp_starttag) > -1:
            line = line[line.find(cdp_starttag):]
            if line.find(cdp_starttag) > -1 and line.find(cdp_endtag) > line.find(cdp_starttag):
                if startwrote is None:
                    startwrote = float(line[line.find(cdp_starttag) + len(cdp_starttag):line.find(cdp_endtag)])
                    logging.debug('wrotestart: %s', str(startwrote))
                else:
                    thiswrote = float(line[line.find(cdp_starttag) + len(cdp_starttag):line.find(cdp_endtag)])
                    endwrote = (endsec - startsec) * 1176 + startwrote
                    this_progress = (thiswrote - startwrote) / (endwrote - startwrote)
                    if progress_callback is not None and progress != this_progress:
                        progress = this_progress
                        progress_callback(progress, *args, **kwargs)
                line = line[line.find(cdp_endtag) + len(cdp_endtag):]
    p.wait()
    if progress != 1:
        progress_callback(1, *args, **kwargs)
