#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
"""
multimedia (Multimedia Module)
==============================

**Author:**

* Dirk Alders <sudo-dirk@mount-mockery.de>

**Description:**

    This module supports converting functions, converting classes and supports getting properties (e.g.: thumbnail, time, ...) from pictures or videos

**Submodules:**

* :mod:`mmod.module.sub1`
* :class:`mmod.module.sub2`
* :func:`mmod.module.sub2`

**Unittest:**

    See also the :download:`unittest <../../multimedia/_testresults_/unittest.pdf>` documentation.
"""
import os
import report
__DEPENDENCIES__ = ['caching', 'fstools', 'report']

__DESCRIPTION__ = """The Module {\\tt %s} supports converting functions, converting classes and supports getting properties (e.g.: thumbnail, time, ...) from pictures or videos.
For more Information read the documentation.""" % __name__.replace('_', '\_')
"""The Module Description"""
__INTERPRETER__ = (2, )
"""The Tested Interpreter-Versions"""


__all__ = ['multimedia.video', 'multimedia.picture']


class base_info(report.logit):
    """Base class to hold and handle information of a file

    :param str filename: Name of the source file

    .. note:: This is is not designed to be used directly.
    """
    TAG_TYPES = {}
    LOG_PREFIX = 'MULTIMEDIA:'

    def __init__(self, filename):
        self.filename = filename
        self._info = None

    def get(self, key, default=None, logger=None):
        """
        This gets an information of the sourcefile by a key.

        :param key: The key (name) of the information to get
        :param default: The default value to be returned, if no information with that key exists
        :returns: The information for the given key
        """
        self.logit_debug(logger, "Property request (%s) for %s", key, os.path.basename(self.filename))
        if self._info is None:
            self._get_info(logger=logger)
        return self._info.get(key, default)

    def keys(self):
        """
        This returns a list of all available keys.

        :returns: The keylist
        :rtype: list
        """
        return self.TAG_TYPES.keys()

    def _get_info(self, logger=None):
        (logger)
        self._info = dict()
